@@ -1,6 +1,4 @@
 """Functions to visualize matrices of data."""
-from __future__ import division
-import itertools
 import warnings
 
 import matplotlib as mpl
@@ -9,12 +7,22 @@ import matplotlib.pyplot as plt
 from matplotlib import gridspec
 import numpy as np
 import pandas as pd
-from scipy.cluster import hierarchy
+try:
+    from scipy.cluster import hierarchy
+    _no_scipy = False
+except ImportError:
+    _no_scipy = True
 
 from . import cm
 from .axisgrid import Grid
-from .utils import (despine, axis_ticklabels_overlap, relative_luminance,
-                    to_utf8)
+from ._compat import get_colormap
+from .utils import (
+    despine,
+    axis_ticklabels_overlap,
+    relative_luminance,
+    to_utf8,
+    _draw_figure,
+)
 
 
 __all__ = ["heatmap", "clustermap"]
@@ -38,26 +46,19 @@ def _index_to_ticklabels(index):
 
 def _convert_colors(colors):
     """Convert either a list of colors or nested lists of colors to RGB."""
-    to_rgb = mpl.colors.colorConverter.to_rgb
+    to_rgb = mpl.colors.to_rgb
 
-    if isinstance(colors, pd.DataFrame):
-        # Convert dataframe
-        return pd.DataFrame({col: colors[col].map(to_rgb)
-                            for col in colors})
-    elif isinstance(colors, pd.Series):
-        return colors.map(to_rgb)
-    else:
-        try:
-            to_rgb(colors[0])
-            # If this works, there is only one level of colors
-            return list(map(to_rgb, colors))
-        except ValueError:
-            # If we get here, we have nested lists
-            return [list(map(to_rgb, l)) for l in colors]
+    try:
+        to_rgb(colors[0])
+        # If this works, there is only one level of colors
+        return list(map(to_rgb, colors))
+    except ValueError:
+        # If we get here, we have nested lists
+        return [list(map(to_rgb, l)) for l in colors]
 
 
 def _matrix_mask(data, mask):
-    """Ensure that data and mask are compatabile and add missing values.
+    """Ensure that data and mask are compatible and add missing values.
 
     Values will be plotted for cells where ``mask`` is ``False``.
 
@@ -66,7 +67,7 @@ def _matrix_mask(data, mask):
 
     """
     if mask is None:
-        mask = np.zeros(data.shape, np.bool)
+        mask = np.zeros(data.shape, bool)
 
     if isinstance(mask, np.ndarray):
         # For array masks, ensure that shape matches data then convert
@@ -76,7 +77,7 @@ def _matrix_mask(data, mask):
         mask = pd.DataFrame(mask,
                             index=data.index,
                             columns=data.columns,
-                            dtype=np.bool)
+                            dtype=bool)
 
     elif isinstance(mask, pd.DataFrame):
         # For DataFrame masks, ensure that semantic labels match data
@@ -93,7 +94,7 @@ def _matrix_mask(data, mask):
     return mask
 
 
-class _HeatMapper(object):
+class _HeatMapper:
     """Draw a heatmap plot of a matrix with nice labels and colormaps."""
 
     def __init__(self, data, vmin, vmax, cmap, center, robust, annot, fmt,
@@ -108,7 +109,7 @@ class _HeatMapper(object):
             plot_data = np.asarray(data)
             data = pd.DataFrame(plot_data)
 
-        # Validate the mask and convet to DataFrame
+        # Validate the mask and convert to DataFrame
         mask = _matrix_mask(data, mask)
 
         plot_data = np.ma.masked_where(np.asarray(mask), plot_data)
@@ -132,9 +133,6 @@ class _HeatMapper(object):
         elif yticklabels is False:
             yticklabels = []
 
-        # Get the positions and used label for the ticks
-        nx, ny = data.T.shape
-
         if not len(xticklabels):
             self.xticks = []
             self.xticklabels = []
@@ -194,11 +192,19 @@ class _HeatMapper(object):
     def _determine_cmap_params(self, plot_data, vmin, vmax,
                                cmap, center, robust):
         """Use some heuristics to set good defaults for colorbar and range."""
-        calc_data = plot_data.data[~np.isnan(plot_data.data)]
+
+        # plot_data is a np.ma.array instance
+        calc_data = plot_data.astype(float).filled(np.nan)
         if vmin is None:
-            vmin = np.percentile(calc_data, 2) if robust else calc_data.min()
+            if robust:
+                vmin = np.nanpercentile(calc_data, 2)
+            else:
+                vmin = np.nanmin(calc_data)
         if vmax is None:
-            vmax = np.percentile(calc_data, 98) if robust else calc_data.max()
+            if robust:
+                vmax = np.nanpercentile(calc_data, 98)
+            else:
+                vmax = np.nanmax(calc_data)
         self.vmin, self.vmax = vmin, vmax
 
         # Choose default colormaps if not provided
@@ -208,7 +214,7 @@ class _HeatMapper(object):
             else:
                 self.cmap = cm.icefire
         elif isinstance(cmap, str):
-            self.cmap = mpl.cm.get_cmap(cmap)
+            self.cmap = get_colormap(cmap)
         elif isinstance(cmap, list):
             self.cmap = mpl.colors.ListedColormap(cmap)
         else:
@@ -216,11 +222,29 @@ class _HeatMapper(object):
 
         # Recenter a divergent colormap
         if center is not None:
+
+            # Copy bad values
+            # in mpl<3.2 only masked values are honored with "bad" color spec
+            # (see https://github.com/matplotlib/matplotlib/pull/14257)
+            bad = self.cmap(np.ma.masked_invalid([np.nan]))[0]
+
+            # under/over values are set for sure when cmap extremes
+            # do not map to the same color as +-inf
+            under = self.cmap(-np.inf)
+            over = self.cmap(np.inf)
+            under_set = under != self.cmap(0)
+            over_set = over != self.cmap(self.cmap.N - 1)
+
             vrange = max(vmax - center, center - vmin)
             normlize = mpl.colors.Normalize(center - vrange, center + vrange)
             cmin, cmax = normlize([vmin, vmax])
             cc = np.linspace(cmin, cmax, 256)
             self.cmap = mpl.colors.ListedColormap(self.cmap(cc))
+            self.cmap.set_bad(bad)
+            if under_set:
+                self.cmap.set_under(under)
+            if over_set:
+                self.cmap.set_over(over)
 
     def _annotate_heatmap(self, ax, mesh):
         """Add textual labels with the value in each cell."""
@@ -272,9 +296,14 @@ class _HeatMapper(object):
         # Remove all the Axes spines
         despine(ax=ax, left=True, bottom=True)
 
+        # setting vmin/vmax in addition to norm is deprecated
+        # so avoid setting if norm is set
+        if "norm" not in kws:
+            kws.setdefault("vmin", self.vmin)
+            kws.setdefault("vmax", self.vmax)
+
         # Draw the heatmap
-        mesh = ax.pcolormesh(self.plot_data, vmin=self.vmin, vmax=self.vmax,
-                             cmap=self.cmap, **kws)
+        mesh = ax.pcolormesh(self.plot_data, cmap=self.cmap, **kws)
 
         # Set the axis limits
         ax.set(xlim=(0, self.data.shape[1]), ylim=(0, self.data.shape[0]))
@@ -305,10 +334,11 @@ class _HeatMapper(object):
         ax.set(xticks=xticks, yticks=yticks)
         xtl = ax.set_xticklabels(xticklabels)
         ytl = ax.set_yticklabels(yticklabels, rotation="vertical")
+        plt.setp(ytl, va="center")  # GH2484
 
         # Possibly rotate them if they overlap
-        if hasattr(ax.figure.canvas, "get_renderer"):
-            ax.figure.draw(ax.figure.canvas.get_renderer())
+        _draw_figure(ax.figure)
+
         if axis_ticklabels_overlap(xtl):
             plt.setp(xtl, rotation="vertical")
         if axis_ticklabels_overlap(ytl):
@@ -322,12 +352,16 @@ class _HeatMapper(object):
             self._annotate_heatmap(ax, mesh)
 
 
-def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
-            annot=None, fmt=".2g", annot_kws=None,
-            linewidths=0, linecolor="white",
-            cbar=True, cbar_kws=None, cbar_ax=None,
-            square=False, xticklabels="auto", yticklabels="auto",
-            mask=None, ax=None, **kwargs):
+def heatmap(
+    data, *,
+    vmin=None, vmax=None, cmap=None, center=None, robust=False,
+    annot=None, fmt=".2g", annot_kws=None,
+    linewidths=0, linecolor="white",
+    cbar=True, cbar_kws=None, cbar_ax=None,
+    square=False, xticklabels="auto", yticklabels="auto",
+    mask=None, ax=None,
+    **kwargs
+):
     """Plot rectangular data as a color-encoded matrix.
 
     This is an Axes-level function and will draw the heatmap into the
@@ -348,7 +382,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         The mapping from data values to color space. If not provided, the
         default will depend on whether ``center`` is set.
     center : float, optional
-        The value at which to center the colormap when plotting divergant data.
+        The value at which to center the colormap when plotting divergent data.
         Using this parameter will change the default ``cmap`` if none is
         specified.
     robust : bool, optional
@@ -358,22 +392,23 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         If True, write the data value in each cell. If an array-like with the
         same shape as ``data``, then use this to annotate the heatmap instead
         of the data. Note that DataFrames will match on position, not index.
-    fmt : string, optional
+    fmt : str, optional
         String formatting code to use when adding annotations.
     annot_kws : dict of key, value mappings, optional
-        Keyword arguments for ``ax.text`` when ``annot`` is True.
+        Keyword arguments for :meth:`matplotlib.axes.Axes.text` when ``annot``
+        is True.
     linewidths : float, optional
         Width of the lines that will divide each cell.
     linecolor : color, optional
         Color of the lines that will divide each cell.
-    cbar : boolean, optional
+    cbar : bool, optional
         Whether to draw a colorbar.
     cbar_kws : dict of key, value mappings, optional
-        Keyword arguments for `fig.colorbar`.
+        Keyword arguments for :meth:`matplotlib.figure.Figure.colorbar`.
     cbar_ax : matplotlib Axes, optional
         Axes in which to draw the colorbar, otherwise take space from the
         main Axes.
-    square : boolean, optional
+    square : bool, optional
         If True, set the Axes aspect to "equal" so each cell will be
         square-shaped.
     xticklabels, yticklabels : "auto", bool, list-like, or int, optional
@@ -381,7 +416,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         the column names. If list-like, plot these alternate labels as the
         xticklabels. If an integer, use the column names but plot only every
         n label. If "auto", try to densely plot non-overlapping labels.
-    mask : boolean array or DataFrame, optional
+    mask : bool array or DataFrame, optional
         If passed, data will not be shown in cells where ``mask`` is True.
         Cells with missing values are automatically masked.
     ax : matplotlib Axes, optional
@@ -389,121 +424,22 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         Axes.
     kwargs : other keyword arguments
         All other keyword arguments are passed to
-        :func:`matplotlib.axes.Axes.pcolormesh`.
+        :meth:`matplotlib.axes.Axes.pcolormesh`.
 
     Returns
     -------
     ax : matplotlib Axes
         Axes object with the heatmap.
 
-    See also
+    See Also
     --------
-    clustermap : Plot a matrix using hierachical clustering to arrange the
+    clustermap : Plot a matrix using hierarchical clustering to arrange the
                  rows and columns.
 
     Examples
     --------
 
-    Plot a heatmap for a numpy array:
-
-    .. plot::
-        :context: close-figs
-
-        >>> import numpy as np; np.random.seed(0)
-        >>> import seaborn as sns; sns.set()
-        >>> uniform_data = np.random.rand(10, 12)
-        >>> ax = sns.heatmap(uniform_data)
-
-    Change the limits of the colormap:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(uniform_data, vmin=0, vmax=1)
-
-    Plot a heatmap for data centered on 0 with a diverging colormap:
-
-    .. plot::
-        :context: close-figs
-
-        >>> normal_data = np.random.randn(10, 12)
-        >>> ax = sns.heatmap(normal_data, center=0)
-
-    Plot a dataframe with meaningful row and column labels:
-
-    .. plot::
-        :context: close-figs
-
-        >>> flights = sns.load_dataset("flights")
-        >>> flights = flights.pivot("month", "year", "passengers")
-        >>> ax = sns.heatmap(flights)
-
-    Annotate each cell with the numeric value using integer formatting:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(flights, annot=True, fmt="d")
-
-    Add lines between each cell:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(flights, linewidths=.5)
-
-    Use a different colormap:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(flights, cmap="YlGnBu")
-
-    Center the colormap at a specific value:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(flights, center=flights.loc["January", 1955])
-
-    Plot every other column label and don't plot row labels:
-
-    .. plot::
-        :context: close-figs
-
-        >>> data = np.random.randn(50, 20)
-        >>> ax = sns.heatmap(data, xticklabels=2, yticklabels=False)
-
-    Don't draw a colorbar:
-
-    .. plot::
-        :context: close-figs
-
-        >>> ax = sns.heatmap(flights, cbar=False)
-
-    Use different axes for the colorbar:
-
-    .. plot::
-        :context: close-figs
-
-        >>> grid_kws = {"height_ratios": (.9, .05), "hspace": .3}
-        >>> f, (ax, cbar_ax) = plt.subplots(2, gridspec_kw=grid_kws)
-        >>> ax = sns.heatmap(flights, ax=ax,
-        ...                  cbar_ax=cbar_ax,
-        ...                  cbar_kws={"orientation": "horizontal"})
-
-    Use a mask to plot only part of a matrix
-
-    .. plot::
-        :context: close-figs
-
-        >>> corr = np.corrcoef(np.random.randn(10, 200))
-        >>> mask = np.zeros_like(corr)
-        >>> mask[np.triu_indices_from(mask)] = True
-        >>> with sns.axes_style("white"):
-        ...     f, ax = plt.subplots(figsize=(7, 5))
-        ...     ax = sns.heatmap(corr, mask=mask, vmax=.3, square=True)
-
+    .. include:: ../docstrings/heatmap.rst
 
     """
     # Initialize the plotter object
@@ -524,7 +460,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
     return ax
 
 
-class _DendrogramPlotter(object):
+class _DendrogramPlotter:
     """Object for drawing tree of similarities between data rows/columns"""
 
     def __init__(self, data, linkage, metric, method, axis, label, rotate):
@@ -657,7 +593,7 @@ class _DendrogramPlotter(object):
         """
         tree_kws = {} if tree_kws is None else tree_kws.copy()
         tree_kws.setdefault("linewidths", .5)
-        tree_kws.setdefault("colors", ".2")
+        tree_kws.setdefault("colors", tree_kws.pop("color", (.2, .2, .2)))
 
         if self.rotate and self.axis == 0:
             coords = zip(self.dependent_coord, self.independent_coord)
@@ -694,8 +630,8 @@ class _DendrogramPlotter(object):
         ytl = ax.set_yticklabels(self.yticklabels, rotation='vertical')
 
         # Force a draw of the plot to avoid matplotlib window error
-        if hasattr(ax.figure.canvas, "get_renderer"):
-            ax.figure.draw(ax.figure.canvas.get_renderer())
+        _draw_figure(ax.figure)
+
         if len(ytl) > 0 and axis_ticklabels_overlap(ytl):
             plt.setp(ytl, rotation="horizontal")
         if len(xtl) > 0 and axis_ticklabels_overlap(xtl):
@@ -703,8 +639,11 @@ class _DendrogramPlotter(object):
         return self
 
 
-def dendrogram(data, linkage=None, axis=1, label=True, metric='euclidean',
-               method='average', rotate=False, tree_kws=None, ax=None):
+def dendrogram(
+    data, *,
+    linkage=None, axis=1, label=True, metric='euclidean',
+    method='average', rotate=False, tree_kws=None, ax=None
+):
     """Draw a tree diagram of relationships within a matrix
 
     Parameters
@@ -742,6 +681,9 @@ def dendrogram(data, linkage=None, axis=1, label=True, metric='euclidean',
     dendrogramplotter.reordered_ind
 
     """
+    if _no_scipy:
+        raise RuntimeError("dendrogram requires scipy to be installed")
+
     plotter = _DendrogramPlotter(data, linkage=linkage, axis=axis,
                                  metric=metric, method=method,
                                  label=label, rotate=rotate)
@@ -757,6 +699,8 @@ class ClusterGrid(Grid):
                  figsize=None, row_colors=None, col_colors=None, mask=None,
                  dendrogram_ratio=None, colors_ratio=None, cbar_pos=None):
         """Grid object for organizing clustered heatmap input on to axes"""
+        if _no_scipy:
+            raise RuntimeError("ClusterGrid requires scipy to be available")
 
         if isinstance(data, pd.DataFrame):
             self.data = data
@@ -768,7 +712,7 @@ class ClusterGrid(Grid):
 
         self.mask = _matrix_mask(self.data2d, mask)
 
-        self.fig = plt.figure(figsize=figsize)
+        self._figure = plt.figure(figsize=figsize)
 
         self.row_colors, self.row_color_labels = \
             self._preprocess_colors(data, row_colors, axis=0)
@@ -799,8 +743,8 @@ class ClusterGrid(Grid):
                                     width_ratios=width_ratios,
                                     height_ratios=height_ratios)
 
-        self.ax_row_dendrogram = self.fig.add_subplot(self.gs[-1, 0])
-        self.ax_col_dendrogram = self.fig.add_subplot(self.gs[0, -1])
+        self.ax_row_dendrogram = self._figure.add_subplot(self.gs[-1, 0])
+        self.ax_col_dendrogram = self._figure.add_subplot(self.gs[0, -1])
         self.ax_row_dendrogram.set_axis_off()
         self.ax_col_dendrogram.set_axis_off()
 
@@ -808,20 +752,20 @@ class ClusterGrid(Grid):
         self.ax_col_colors = None
 
         if self.row_colors is not None:
-            self.ax_row_colors = self.fig.add_subplot(
+            self.ax_row_colors = self._figure.add_subplot(
                 self.gs[-1, 1])
         if self.col_colors is not None:
-            self.ax_col_colors = self.fig.add_subplot(
+            self.ax_col_colors = self._figure.add_subplot(
                 self.gs[1, -1])
 
-        self.ax_heatmap = self.fig.add_subplot(self.gs[-1, -1])
+        self.ax_heatmap = self._figure.add_subplot(self.gs[-1, -1])
         if cbar_pos is None:
             self.ax_cbar = self.cax = None
         else:
             # Initialize the colorbar axes in the gridspec so that tight_layout
             # works. We will move it where it belongs later. This is a hack.
-            self.ax_cbar = self.fig.add_subplot(self.gs[0, 0])
-            self.cax = self.ax_cbar  # Backwards compatability
+            self.ax_cbar = self._figure.add_subplot(self.gs[0, 0])
+            self.cax = self.ax_cbar  # Backwards compatibility
         self.cbar_pos = cbar_pos
 
         self.dendrogram_row = None
@@ -833,15 +777,26 @@ class ClusterGrid(Grid):
 
         if colors is not None:
             if isinstance(colors, (pd.DataFrame, pd.Series)):
+
+                # If data is unindexed, raise
+                if (not hasattr(data, "index") and axis == 0) or (
+                    not hasattr(data, "columns") and axis == 1
+                ):
+                    axis_name = "col" if axis else "row"
+                    msg = (f"{axis_name}_colors indices can't be matched with data "
+                           f"indices. Provide {axis_name}_colors as a non-indexed "
+                           "datatype, e.g. by using `.to_numpy()``")
+                    raise TypeError(msg)
+
                 # Ensure colors match data indices
                 if axis == 0:
                     colors = colors.reindex(data.index)
                 else:
                     colors = colors.reindex(data.columns)
 
-                # Replace na's with background color
+                # Replace na's with white color
                 # TODO We should set these to transparent instead
-                colors = colors.fillna('white')
+                colors = colors.astype(object).fillna('white')
 
                 # Extract color values and labels from frame/series
                 if isinstance(colors, pd.DataFrame):
@@ -919,9 +874,6 @@ class ClusterGrid(Grid):
         axis : int
             Which axis to normalize across. If 0, normalize across rows, if 1,
             normalize across columns.
-        vmin : int
-            If 0, then subtract the minimum of the data before dividing by
-            the range.
 
         Returns
         -------
@@ -950,7 +902,7 @@ class ClusterGrid(Grid):
         ratios = [dendrogram_ratio]
 
         if colors is not None:
-            # Colors are encoded as rgb, so ther is an extra dimention
+            # Colors are encoded as rgb, so there is an extra dimension
             if np.ndim(colors) > 2:
                 n_colors = len(colors)
             else:
@@ -983,42 +935,38 @@ class ClusterGrid(Grid):
         Returns
         -------
         matrix : numpy.array
-            A numpy array of integer values, where each corresponds to a color
-            from the originally provided list of colors
+            A numpy array of integer values, where each indexes into the cmap
         cmap : matplotlib.colors.ListedColormap
 
         """
-        # check for nested lists/color palettes.
-        # Will fail if matplotlib color is list not tuple
-        if any(issubclass(type(x), list) for x in colors):
-            all_colors = set(itertools.chain(*colors))
-            n = len(colors)
-            m = len(colors[0])
+        try:
+            mpl.colors.to_rgb(colors[0])
+        except ValueError:
+            # We have a 2D color structure
+            m, n = len(colors), len(colors[0])
+            if not all(len(c) == n for c in colors[1:]):
+                raise ValueError("Multiple side color vectors must have same size")
         else:
-            all_colors = set(colors)
-            n = 1
-            m = len(colors)
+            # We have one vector of colors
+            m, n = 1, len(colors)
             colors = [colors]
-        color_to_value = dict((col, i) for i, col in enumerate(all_colors))
 
-        matrix = np.array([color_to_value[c]
-                           for color in colors for c in color])
+        # Map from unique colors to colormap index value
+        unique_colors = {}
+        matrix = np.zeros((m, n), int)
+        for i, inner in enumerate(colors):
+            for j, color in enumerate(inner):
+                idx = unique_colors.setdefault(color, len(unique_colors))
+                matrix[i, j] = idx
 
-        shape = (n, m)
-        matrix = matrix.reshape(shape)
+        # Reorder for clustering and transpose for axis
         matrix = matrix[:, ind]
         if axis == 0:
-            # row-side:
             matrix = matrix.T
 
-        cmap = mpl.colors.ListedColormap(all_colors)
+        cmap = mpl.colors.ListedColormap(list(unique_colors))
         return matrix, cmap
 
-    def savefig(self, *args, **kwargs):
-        if 'bbox_inches' not in kwargs:
-            kwargs['bbox_inches'] = 'tight'
-        self.fig.savefig(*args, **kwargs)
-
     def plot_dendrograms(self, row_cluster, col_cluster, metric, method,
                          row_linkage, col_linkage, tree_kws):
         # Plot the row dendrogram
@@ -1127,7 +1075,7 @@ class ClusterGrid(Grid):
 
         # Reorganize the annotations to match the heatmap
         annot = kws.pop("annot", None)
-        if annot is None:
+        if annot is None or annot is False:
             pass
         else:
             if isinstance(annot, bool):
@@ -1156,13 +1104,13 @@ class ClusterGrid(Grid):
 
         tight_params = dict(h_pad=.02, w_pad=.02)
         if self.ax_cbar is None:
-            self.fig.tight_layout(**tight_params)
+            self._figure.tight_layout(**tight_params)
         else:
             # Turn the colorbar axes off for tight layout so that its
             # ticks don't interfere with the rest of the plot layout.
             # Then move it.
             self.ax_cbar.set_axis_off()
-            self.fig.tight_layout(**tight_params)
+            self._figure.tight_layout(**tight_params)
             self.ax_cbar.set_axis_on()
             self.ax_cbar.set_position(self.cbar_pos)
 
@@ -1195,34 +1143,39 @@ class ClusterGrid(Grid):
         return self
 
 
-def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
-               z_score=None, standard_scale=None, figsize=(10, 10),
-               cbar_kws=None, row_cluster=True, col_cluster=True,
-               row_linkage=None, col_linkage=None,
-               row_colors=None, col_colors=None, mask=None,
-               dendrogram_ratio=.2, colors_ratio=0.03,
-               cbar_pos=(.02, .8, .05, .18), tree_kws=None,
-               **kwargs):
-    """Plot a matrix dataset as a hierarchically-clustered heatmap.
+def clustermap(
+    data, *,
+    pivot_kws=None, method='average', metric='euclidean',
+    z_score=None, standard_scale=None, figsize=(10, 10),
+    cbar_kws=None, row_cluster=True, col_cluster=True,
+    row_linkage=None, col_linkage=None,
+    row_colors=None, col_colors=None, mask=None,
+    dendrogram_ratio=.2, colors_ratio=0.03,
+    cbar_pos=(.02, .8, .05, .18), tree_kws=None,
+    **kwargs
+):
+    """
+    Plot a matrix dataset as a hierarchically-clustered heatmap.
+
+    This function requires scipy to be available.
 
     Parameters
     ----------
-    data: 2D array-like
+    data : 2D array-like
         Rectangular data for clustering. Cannot contain NAs.
     pivot_kws : dict, optional
         If `data` is a tidy dataframe, can provide keyword arguments for
         pivot to create a rectangular dataframe.
     method : str, optional
-        Linkage method to use for calculating clusters.
-        See scipy.cluster.hierarchy.linkage documentation for more information:
-        https://docs.scipy.org/doc/scipy/reference/generated/scipy.cluster.hierarchy.linkage.html
+        Linkage method to use for calculating clusters. See
+        :func:`scipy.cluster.hierarchy.linkage` documentation for more
+        information.
     metric : str, optional
         Distance metric to use for the data. See
-        scipy.spatial.distance.pdist documentation for more options
-        https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.pdist.html
+        :func:`scipy.spatial.distance.pdist` documentation for more options.
         To use different metrics (or methods) for rows and columns, you may
         construct each linkage matrix yourself and provide them as
-        {row,col}_linkage.
+        `{row,col}_linkage`.
     z_score : int or None, optional
         Either 0 (rows) or 1 (columns). Whether or not to calculate z-scores
         for the rows or the columns. Z scores are: z = (x - mean)/std, so
@@ -1233,44 +1186,48 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
         Either 0 (rows) or 1 (columns). Whether or not to standardize that
         dimension, meaning for each row or column, subtract the minimum and
         divide each by its maximum.
-    figsize: (width, height), optional
+    figsize : tuple of (width, height), optional
         Overall size of the figure.
     cbar_kws : dict, optional
-        Keyword arguments to pass to ``cbar_kws`` in ``heatmap``, e.g. to
+        Keyword arguments to pass to `cbar_kws` in :func:`heatmap`, e.g. to
         add a label to the colorbar.
     {row,col}_cluster : bool, optional
-        If True, cluster the {rows, columns}.
-    {row,col}_linkage : numpy.array, optional
+        If ``True``, cluster the {rows, columns}.
+    {row,col}_linkage : :class:`numpy.ndarray`, optional
         Precomputed linkage matrix for the rows or columns. See
-        scipy.cluster.hierarchy.linkage for specific formats.
+        :func:`scipy.cluster.hierarchy.linkage` for specific formats.
     {row,col}_colors : list-like or pandas DataFrame/Series, optional
-        List of colors to label for either the rows or columns. Useful to
-        evaluate whether samples within a group are clustered together. Can
-        use nested lists or DataFrame for multiple color levels of labeling.
-        If given as a DataFrame or Series, labels for the colors are extracted
-        from the DataFrames column names or from the name of the Series.
-        DataFrame/Series colors are also matched to the data by their
-        index, ensuring colors are drawn in the correct order.
-    mask : boolean array or DataFrame, optional
-        If passed, data will not be shown in cells where ``mask`` is True.
+        List of colors to label for either the rows or columns. Useful to evaluate
+        whether samples within a group are clustered together. Can use nested lists or
+        DataFrame for multiple color levels of labeling. If given as a
+        :class:`pandas.DataFrame` or :class:`pandas.Series`, labels for the colors are
+        extracted from the DataFrames column names or from the name of the Series.
+        DataFrame/Series colors are also matched to the data by their index, ensuring
+        colors are drawn in the correct order.
+    mask : bool array or DataFrame, optional
+        If passed, data will not be shown in cells where `mask` is True.
         Cells with missing values are automatically masked. Only used for
         visualizing, not for calculating.
-    {dendrogram,colors}_ratio: float, or pair of floats, optional
+    {dendrogram,colors}_ratio : float, or pair of floats, optional
         Proportion of the figure size devoted to the two marginal elements. If
         a pair is given, they correspond to (row, col) ratios.
-    cbar_pos : (left, bottom, width, height), optional
+    cbar_pos : tuple of (left, bottom, width, height), optional
         Position of the colorbar axes in the figure. Setting to ``None`` will
         disable the colorbar.
     tree_kws : dict, optional
         Parameters for the :class:`matplotlib.collections.LineCollection`
         that is used to plot the lines of the dendrogram tree.
     kwargs : other keyword arguments
-        All other keyword arguments are passed to :func:`heatmap`
+        All other keyword arguments are passed to :func:`heatmap`.
 
     Returns
     -------
-    clustergrid : ClusterGrid
-        A ClusterGrid instance.
+    :class:`ClusterGrid`
+        A :class:`ClusterGrid` instance.
+
+    See Also
+    --------
+    heatmap : Plot rectangular data as a color-encoded matrix.
 
     Notes
     -----
@@ -1286,73 +1243,12 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
     Examples
     --------
 
-    Plot a clustered heatmap:
-
-    .. plot::
-        :context: close-figs
-
-        >>> import seaborn as sns; sns.set(color_codes=True)
-        >>> iris = sns.load_dataset("iris")
-        >>> species = iris.pop("species")
-        >>> g = sns.clustermap(iris)
-
-    Change the size and layout of the figure:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris,
-        ...                    figsize=(7, 5),
-        ...                    row_cluster=False,
-        ...                    dendrogram_ratio=(.1, .2),
-        ...                    cbar_pos=(0, .2, .03, .4))
-
-    Add colored labels to identify observations:
-
-    .. plot::
-        :context: close-figs
-
-        >>> lut = dict(zip(species.unique(), "rbg"))
-        >>> row_colors = species.map(lut)
-        >>> g = sns.clustermap(iris, row_colors=row_colors)
-
-    Use a different colormap and adjust the limits of the color range:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris, cmap="mako", vmin=0, vmax=10)
-
-    Use a different similarity metric:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris, metric="correlation")
-
-    Use a different clustering method:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris, method="single")
-
-    Standardize the data within the columns:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris, standard_scale=1)
-
-    Normalize the data within the rows:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.clustermap(iris, z_score=0, cmap="vlag")
-
+    .. include:: ../docstrings/clustermap.rst
 
     """
+    if _no_scipy:
+        raise RuntimeError("clustermap requires scipy to be available")
+
     plotter = ClusterGrid(data, pivot_kws=pivot_kws, figsize=figsize,
                           row_colors=row_colors, col_colors=col_colors,
                           z_score=z_score, standard_scale=standard_scale,
